<?php

/**
 * Builds forms on block send dashboard.
 *
 * @param string $type
 *   The type of task we are working on.
 *
 * @return array
 *   Array of forms for the block send dashboard.
 */
function globallink_block_dashboard($type) {
  $default_date = format_date(REQUEST_TIME + (7 * 24 * 60 * 60), 'custom', 'm/d/Y');

  drupal_add_library('system', 'ui.datepicker');
  drupal_add_js('jQuery(function() {
    jQuery(document).ajaxComplete(function() {
      jQuery(".pickadate1").datepicker({
        dateFormat: "mm/dd/yy",
        defaultDate: "' . $default_date . '",
        minDate: 0
      });
    });
  });', array('group' => CSS_THEME, 'type' => 'inline', 'scope' => 'header', 'weight' => 80));
  
  $_SESSION['globallink_selected_type'] = $type;

  $array = array();

  $array[] = drupal_get_form('globallink_block_dashboard_filter_form');
  $array[] = drupal_get_form('globallink_block_dashboard_pager_form');
  $array[] = drupal_get_form('globallink_block_dashboard_form');

  return $array;
}

/**
 * Builds form to filter blocks to send for translation on dashboard.
 */
function globallink_block_dashboard_filter_form() {
  module_load_include('inc', 'globallink', 'globallink');

  $form = array();

  $locales = globallink_get_mapped_drupal_locales(FALSE);

  if (isset($locales) && count($locales) > 1) {
    $filter = FALSE;
    $default_info = '';

    if (!empty($_SESSION['globallink_selected_block_info'])) {
      $default_info = $_SESSION['globallink_selected_block_info'];
      $filter = TRUE;
    }

    $form['block_info_filter'] = array(
      '#type' => 'fieldset',
      '#title' => t('FILTER BLOCKS'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );

    $form['block_info_filter']['filters'] = array(
      '#type' => 'container',
      '#attributes' => array('class' => array('container-inline')),
    );

    $form['block_info_filter']['filters']['info'] = array(
      '#type' => 'textfield',
      '#title' => 'Block Info' . ':&nbsp;&nbsp;',
      '#attributes' => array('size' => 30),
      '#default_value' => $default_info,
      '#field_suffix' => '&nbsp;&nbsp;&nbsp;&nbsp;',
    );

    $disable_submit = $filter ? TRUE : FALSE;

    $form['block_info_filter']['filters']['filter'] = array(
      '#type' => 'submit',
      '#value' => t('Filter'),
      '#disabled' => $disable_submit,
    );

    if ($filter) {
      $form['filter-search'] = array(
        '#type' => 'fieldset',
        '#title' => t('FILTER CRITERIA'),
      );

      $form['filter-search']['filter-markup'] = array(
        '#type' => 'container',
      );

      $form['filter-search']['filter-markup'][] = array('#markup' => '<table>');

      $form['filter-search']['filter-markup'][] = array(
        '#markup' => '<tr><td style="text-align: left;width: 15%;vertical-align: middle;border: none;"><b>' . 'Block Info' . '</b>:&nbsp;</td><td style="text-align: left;width: 85%;vertical-align: middle;border: none;">' . $default_info . '</td></tr>',
      );

      $form['filter-search']['filter-markup'][] = array('#markup' => '</table>');

      $form['filter-search']['status']['actions'] = array(
        '#type' => 'actions',
        '#attributes' => array('class' => array('container-inline')),
      );

      $form['filter-search']['status']['actions']['reset'] = array(
        '#type' => 'submit',
        '#value' => t('Reset'),
      );
    }
  }
  else {
    $form['locale_mapping']['no_mapping'] = array(
      '#type' => 'markup',
      '#markup' => t('<br/><b><i>No GlobalLink locale mapping found.</i></b>'),
    );
  }

  return $form;
}

/**
 * Handles submission of filter form.
 */
function globallink_block_dashboard_filter_form_submit($form, &$form_state) {
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';

  switch ($op) {
    case t('Filter'):
      $_SESSION['globallink_selected_block_info'] = $form_state['values']['info'];

      break;
    case t('Reset'):
      $_SESSION['globallink_selected_block_info'] = '';

      break;
  }
}

/**
 * Builds form to add pagination to block send dashboard.
 */
function globallink_block_dashboard_pager_form() {
  $form = array();

  $locales = globallink_get_mapped_drupal_locales(FALSE);

  if (isset($locales) && count($locales) > 1) {
    $form['page_counter']['markup'] = array(
      '#type' => 'container',
      '#attributes' => array('class' => array('tpt-center')),
    );

    $form['page_counter']['markup']['p_count'] = array(
      '#type' => 'textfield',
      '#size' => 5,
      '#default_value' => isset($_SESSION['globallink_block_page_count']) ? $_SESSION['globallink_block_page_count'][0] : TPT_PAGER_LIMIT,
      '#field_prefix' => t('Show') . '&nbsp;&nbsp;',
      '#field_suffix' => '&nbsp;&nbsp;' . t('records') . '&nbsp;&nbsp;&nbsp;&nbsp;',
      '#prefix' => '<div class="container-inline">',
    );

    $form['page_counter']['markup']['action'] = array(
      '#type' => 'submit',
      '#value' => t('Go'),
      '#suffix' => '</div>',
      '#limit_validation_errors' => array(),
      '#submit' => array('globallink_block_pager_form_submit'),
      '#access' => TRUE,
    );
  }

  return $form;
}

/**
 * Handles submission of pager form.
 */
function globallink_block_pager_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];

  if ($op != t('Go')) {
    return;
  }

  $page_count = $form_state['input']['p_count'];

  if (!is_numeric($page_count) || $page_count < 1) {
    form_set_error('', t('Invalid Page Count.'));
  }
  else {
    $_SESSION['globallink_block_page_count'] = array($page_count);

    return;
  }
}

/**
 * Builds form to create a block submission.
 */
function globallink_block_dashboard_form() {
  module_load_include('inc', 'globallink_block', 'globallink_block');
  module_load_include('inc', 'globallink', 'globallink');
  module_load_include('inc', 'globallink', 'globallink_send_translations');

  $module_path = drupal_get_path('module', 'globallink');
  drupal_add_js($module_path . '/js/globallink_send_translations.js');
  drupal_add_js(array(
    'globallinkSendTranslations' => array(
      'selectAll' => base_path() . globallink_get_root_menu('create-submission/select_all_js'),
      'selectedPath' => base_path() . globallink_get_root_menu('create-submission/select_id_js'),
      'selectedType' => GLOBALLINK_ENTITY_TYPE_BLOCK,
      'sessionKey' => 'bids',
    ),
  ), 'setting');

  drupal_add_css($module_path . '/css/globallink.css');

  $form = array();

  $default_checked = array();
  if (isset($_SESSION['bids'])) {
    $default_checked = $_SESSION['bids'];
  }  

  $locales = globallink_get_mapped_drupal_locales(FALSE);

  if (isset($locales) && count($locales) > 1) {
    $target_lang_arr = globallink_get_mapped_locales_with_drupal_desc(FALSE);

    $default_info = '';

    if (!empty($_SESSION['globallink_selected_block_info'])) {
      $default_info = $_SESSION['globallink_selected_block_info'];
    }

    $default_language = language_default();
    $default = $default_language->language;

    unset($target_lang_arr[globallink_get_locale_code($default)]);

    $t_count = 0;

    foreach ($target_lang_arr as $key => $value) {
      $t_count++;

      if ($t_count % 2 === 0) {
        $target_lang_arr[$key] = '&nbsp;&nbsp;' . $value . '<BR/>';
      }
      else {
        $target_lang_arr[$key] = '&nbsp;&nbsp;' . $value . '&nbsp;&nbsp;&nbsp;&nbsp;';
      }
    }

    $query = db_select('i18n_string', 'i18n')
      ->condition('textgroup', 'blocks', '=')
        ->fields('i18n', array('objectid'))
      ->distinct();

    $result = $query->execute();

    $block_ids = array();

    foreach ($result as $row) {
      $block_ids[] = $row->objectid;
    }

    $page_count = TPT_PAGER_LIMIT;

    if (isset($_SESSION['globallink_block_page_count'])) {
      $page_count = $_SESSION['globallink_block_page_count'][0];
    }

    $header = array(
      'info' => array('field' => 'bc.info', 'data' => t('Block Info')),
      'active' => t('In Active Submission'),
      'preview' => t('Preview'),
    );

    $rows = array();

    $count = 0;
    if (count($block_ids) > 0) {
      $query = db_select('block_custom', 'bc')
        ->condition('bid', $block_ids, 'IN')
        ->fields('bc')
        ->extend('PagerDefault')
        ->limit($page_count)
        ->extend('TableSort')
        ->orderByHeader($header);

      if ($default_info != '') {
        $query->condition('info', '%' . $default_info . '%', 'LIKE');
      }

      $result = $query->execute();

      foreach ($result as $item) {

        $active = '';
        $active_arr = globallink_block_get_active_submission_by_id($item->bid);

        if (!empty($active_arr)) {
          foreach ($active_arr as $active_row) {
            $l_title = globallink_format_display_string($active_row->submission);
            $l_href = 'admin/globallink-translations/workbench/all/' . $active_row->submission_rid . '/' . $active_row->target_lang_code;
            $l_options = array();
            $active .= l($l_title, $l_href, $l_options) . '&nbsp;&nbsp;- &nbsp;&nbsp;' . $active_row->sub_target_lang_name . ' <BR/> ';
          }
        }

        $rows[$item->bid] = array(
          'info' => $item->info,
          'active' => $active,
          'preview' => globallink_block_make_preview_link('Preview', $item->bid),
        );

        $count++;
      }
    }
    
    $cart_count = get_cart_count();
    
    globallink_add_cart_form_elements($form, $header, $rows, $default_checked, $cart_count);

    if ($count > 0) {
      $form['pager'] = ['#markup' => theme('pager')];
    }
  }

  // Very important
  form_load_include($form_state, 'inc', 'globallink', 'globallink_send_translations');

  return $form;
}

/**
 * Validates block form input.
 */
function globallink_block_dashboard_form_validate($form, &$form_state) {
  return;
}

/**
 * Handles block form submission.
 */
function globallink_block_dashboard_form_submit($form, &$form_state) {
  return;
}

/**
 * Function to create Preview link
 */
function globallink_block_make_preview_link($link_text = '', $id) {
  drupal_add_js(array(
    'my-modal-style' => array(
      'modalSize' => array(
        'type' => 'fixed',
        'width' => 700,
        'height' => 400,
      ),
      'animation' => 'fadeIn',
    ),
      ), 'setting');

  // Set a default value if no text in supplied.
  if (empty($link_text)) {
    $link_text = 'Preview';
  }

  return '<div id="preview-link" style="text-align: left;" title="Click to Preview">' . l($link_text, 'admin/globallink-translations/dashboard/block/preview/' . $id . '/nojs', array('attributes' => array('class' => 'ctools-use-modal ctools-modal-my-modal-style'))) . '</div>';
}

/**
 * Helper function
 */
function globallink_block_preview($ajax) {
  if ($ajax) {
    ctools_include('ajax');
    ctools_include('modal');

    $form_state = array(
      'ajax' => TRUE,
      'title' => t('Preview'),
    );

    // Use ctools to generate ajax instructions for the browser to create a form in a modal popup.
    $output = ctools_modal_form_wrapper('globallink_block_preview_content', $form_state);

    // If the form has been submitted, there may be additional instructions such as dismissing the modal popup.
    if (!empty($form_state['ajax_commands'])) {
      $output = $form_state['ajax_commands'];
    }

    // Return the ajax instructions to the browser via ajax_render().
    print ajax_render($output);
    drupal_exit();
  }
  else {
    return drupal_get_form('globallink_block_preview_content');
  }
}

/**
 * Helper function
 */
function globallink_block_preview_content() {
  $id = arg(5);
  $output = "";

  $query = db_select('block_custom', 'bc')
      ->fields('bc', array('body', 'info'))
      ->condition('bid', $id, '=');

  $result = $query->execute();
  $result = $result->fetchAll();

  if (count($result) > 0) {
    $output .= "<table><tr><th>Fields</th><th>Source Contents</th></tr>";
    foreach ($result as $val) {
      $output .= "<tr><td><strong>Block Info</strong></td><td>" . $val->info . "</td></tr>";
      $output .= "<tr><td><strong>Body</strong></td><td>" . $val->body . "</td></tr>";
    }
    $output .= "</table>";
  }

  $form['preview'] = array(
    '#markup' => $output,
  );

  return $form;
}