<?php

/**
 * Builds forms on webform send dashboard.
 *
 * @param string $type
 *   The type of task we are working on.
 *
 * @return array
 *   Array of forms for the webform send dashboard.
 */
function globallink_webform_dashboard_page($type) {
  $default_date = format_date(REQUEST_TIME + (7 * 24 * 60 * 60), 'custom', 'm/d/Y');

  drupal_add_library('system', 'ui.datepicker');
  drupal_add_js('jQuery(function() {
    jQuery(document).ajaxComplete(function() {
      jQuery(".pickadate1").datepicker({
        dateFormat: "mm/dd/yy",
        defaultDate: "' . $default_date . '",
        minDate: 0
      });
    });
  });', array('group' => CSS_THEME, 'type' => 'inline', 'scope' => 'header', 'weight' => 80));
  
  $_SESSION['globallink_selected_type'] = $type;

  $array = array();

  $array[] = drupal_get_form('globallink_webform_dashboard_filter_form');
  $array[] = drupal_get_form('globallink_webform_dashboard_pager_form');
  $array[] = drupal_get_form('globallink_webform_dashboard_form');

  return $array;
}

/**
 * Builds form to filter webforms to send for translation on dashboard.
 */
function globallink_webform_dashboard_filter_form() {
  module_load_include('inc', 'globallink', 'globallink');

  $form = array();

  $locales = globallink_get_mapped_drupal_locales(FALSE);

  if (isset($locales) && count($locales) > 1) {
    $results = globallink_drupal_search_query();

    $form['filter'] = array(
      '#type' => 'fieldset',
      '#title' => t('FILTER STRINGS'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );

    foreach ($results as $key => $result) {
      $filter = FALSE;

      if ($key == 'string') {
        $form['filter']['string'] = array(
          '#type' => 'textfield',
          '#title' => check_plain($result['title']),
          '#description' => check_plain($result['description']),
        );
      }
      else {
        $form['filter'][$key] = array(
          '#attributes' => array('class' => array('container-inline')),
          '#title' => check_plain($result['title']),
          '#type' => 'select',
          '#empty_value' => 'all',
          '#empty_option' => $result['options']['all'],
          '#options' => $result['options'],
        );

        if (!empty($_SESSION['locale_translation_filter'][$key])) {
          $form['filters']['status'][$key]['#default_value'] = $_SESSION['locale_translation_filter'][$key];
        }
      }

      if (!empty($_SESSION['globallink_webform_filter'][$key])) {
        $form['filter'][$key]['#default_value'] = $_SESSION['globallink_webform_filter'][$key];
        $filter = TRUE;
      }
    }

    $form['filter']['actions'] = array(
      '#type' => 'actions',
    );

    $disable_submit = $filter ? TRUE : FALSE;

    $form['filter']['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Filter'),
      '#disabled' => $disable_submit,
    );

    if ($filter) {
      $form['filter-search'] = array(
        '#type' => 'fieldset',
        '#title' => t('FILTER CRITERIA'),
      );

      $form['filter-search']['filter-markup'] = array(
        '#type' => 'container',
      );

      $form['filter-search']['filter-markup'][] = array(
        '#markup' => '<table>',
      );

      foreach ($_SESSION['globallink_webform_filter'] as $key => $value) {
        switch ($key) {
          case 'string':
            if (empty($value)) {
              continue;
            }

            $form['filter-search']['filter-markup'][] = array(
              '#markup' => '<tr><td style="text-align: left;width: 15%;vertical-align: middle;border: none;"><b>' . 'String contains' . '</b>:&nbsp;</td><td style="text-align: left;width: 85%;vertical-align: middle;border: none;">' . $value . '</td></tr>',
            );

            break;

          case 'language':
            if (empty($value)) {
              continue;
            }

            $form['filter-search']['filter-markup'][] = array(
              '#markup' => '<tr><td style="text-align: left;width: 15%;vertical-align: middle;border: none;"><b>' . 'Language' . '</b>:&nbsp;</td><td style="text-align: left;width: 85%;vertical-align: middle;border: none;">' . $results['language']['options'][$value] . '</td></tr>',
            );

            break;

          case 'translation':
            if (empty($value)) {
              continue;
            }

            $form['filter-search']['filter-markup'][] = array(
              '#markup' => '<tr><td style="text-align: left;width: 15%;vertical-align: middle;border: none;"><b>' . 'Filter in' . '</b>:&nbsp;</td><td style="text-align: left;width: 85%;vertical-align: middle;border: none;">' . $results['translation']['options'][$value] . '</td></tr>',
            );

            break;
        }
      }

      $form['filter-search']['filter-markup'][] = array(
        '#markup' => '</table>',
      );

      $form['filter-search']['status']['actions'] = array(
        '#type' => 'actions',
        '#attributes' => array('class' => array('container-inline')),
      );

      $form['filter-search']['status']['actions']['reset'] = array(
        '#type' => 'submit',
        '#value' => t('Reset'),
      );

      drupal_add_css(drupal_get_path('module', 'globallink') . '/globallink_webform/css/webform.css');
    }
  }
  else {
    $form['locale_mapping']['no_mapping'] = array(
      '#type' => 'markup',
      '#markup' => t('<br/><b><i>No GlobalLink locale mapping found.</i></b>'),
    );
  }

  return $form;
}

/**
 * Handles submission of filter form.
 */
function globallink_webform_dashboard_filter_form_submit($form, &$form_state) {
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';

  switch ($op) {
    case t('Filter'):
      $_SESSION['globallink_webform_filter']['string'] = $form_state['values']['string'];
      $_SESSION['globallink_webform_filter']['language'] = $form_state['values']['language'];
      $_SESSION['globallink_webform_filter']['translation'] = $form_state['values']['translation'];

      break;

    case t('Reset'):
      $_SESSION['globallink_webform_filter']['string'] = '';
      $_SESSION['globallink_webform_filter']['language'] = '';
      $_SESSION['globallink_webform_filter']['translation'] = '';

      break;
  }
}

/**
 * Builds form to add pagination to webform send dashboard.
 */
function globallink_webform_dashboard_pager_form() {
  $form = array();

  $locales = globallink_get_mapped_drupal_locales(FALSE);

  if (isset($locales) && count($locales) > 1) {
    $form['page_counter']['markup'] = array(
      '#type' => 'container',
      '#attributes' => array('class' => array('tpt-center')),
    );

    $form['page_counter']['markup']['p_count'] = array(
      '#type' => 'textfield',
      '#size' => 5,
      '#default_value' => isset($_SESSION['globallink_webform_page_count']) ? $_SESSION['globallink_webform_page_count'][0] : TPT_PAGER_LIMIT,
      '#field_prefix' => t('Show') . '&nbsp;&nbsp;',
      '#field_suffix' => '&nbsp;&nbsp;' . t('records') . '&nbsp;&nbsp;&nbsp;&nbsp;',
      '#prefix' => '<div class="container-inline">',
    );

    $form['page_counter']['markup']['action'] = array(
      '#type' => 'submit',
      '#value' => t('Go'),
      '#suffix' => '</div>',
      '#limit_validation_errors' => array(),
      '#submit' => array('globallink_webform_pager_form_submit'),
      '#access' => TRUE,
    );
  }

  return $form;
}

/**
 * Handles submission of pager form.
 */
function globallink_webform_pager_form_submit($form, &$form_state) {
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';

  if ($op != t('Go')) {
    return;
  }

  $page_count = $form_state['input']['p_count'];
  if (!is_numeric($page_count) || $page_count < 1) {
    form_set_error('', t('Invalid Page Count.'));
  }
  else {
    $_SESSION['globallink_webform_page_count'] = array($page_count);
    return;
  }

}

/**
 * Builds form to create a webform submission.
 */
function globallink_webform_dashboard_form() {
  module_load_include('inc', 'globallink_webform', 'globallink_webform');
  module_load_include('inc', 'globallink', 'globallink');
  module_load_include('inc', 'globallink', 'globallink_send_translations');

  $module_path = drupal_get_path('module', 'globallink');
  drupal_add_js($module_path . '/js/globallink_send_translations.js');
  drupal_add_js(array(
    'globallinkSendTranslations' => array(
      'selectAll' => base_path() . globallink_get_root_menu('create-submission/select_all_js'),
      'selectedPath' => base_path() . globallink_get_root_menu('create-submission/select_id_js'),
      'selectedType' => GLOBALLINK_ENTITY_TYPE_WEBFORM,
      'sessionKey' => 'wfids',
    ),
  ), 'setting');

  drupal_add_css($module_path . '/css/globallink.css');
  
  $form = array();

  $default_checked = array();
  if (isset($_SESSION['wfids'])) {
    $default_checked = $_SESSION['wfids'];
  }
  
  $locales = globallink_get_mapped_drupal_locales(FALSE);

  if (isset($locales) && count($locales) > 1) {
    $target_lang_arr = globallink_get_mapped_locales_with_drupal_desc(FALSE);
    $source_lang_arr = globallink_get_mapped_locales_with_drupal_desc(FALSE);

    $page_count = TPT_PAGER_LIMIT;

    if (isset($_SESSION['globallink_webform_page_count'])) {
      $page_count = $_SESSION['globallink_webform_page_count'][0];
    }

    $default_language = language_default();
    $default = $default_language->language;

    if (!empty($_SESSION['globallink_selected_webform_language'])) {
      $default = $_SESSION['globallink_selected_webform_language'];
    }

    unset($target_lang_arr[globallink_get_locale_code($default)]);

    $t_count = 0;

    foreach ($target_lang_arr as $key => $value) {
      $t_count++;

      if ($t_count % 2 === 0) {
        $target_lang_arr[$key] = '&nbsp;&nbsp;' . $value . '<BR/>';
      }
      else {
        $target_lang_arr[$key] = '&nbsp;&nbsp;' . $value . '&nbsp;&nbsp;&nbsp;&nbsp;';
      }
    }

    if (!($query = globallink_webform_get_translate_filter_query())) {
      $query = array(
        'translation' => 'all',
        'language' => 'all',
        'string' => '',
      );
    }

    $sql_query = db_select('locales_source', 's')
        ->extend('PagerDefault')
        ->limit($page_count);

    $limit_language = NULL;

    if ($query['language'] != 'en' && $query['language'] != 'all') {
      $sql_query->leftJoin('locales_target', 't', 't.lid = s.lid AND t.language = :langcode', array(
        ':langcode' => $query['language'])
      );

      $limit_language = $query['language'];
    }
    else {
      $sql_query->leftJoin('locales_target', 't', 't.lid = s.lid');
    }

    $header = array(
      'parent' => t('WEBFORM TITLE'),
      'string' => array(
        'data' => t('String'),
        'field' => 's.location',
      ),
      'languages' => ($limit_language) ? t('Language') : t('IN ACTIVE SUBMISSION'),
      'preview' => t('Preview'),
    );

    $sql_query->fields('s', array('source', 'location', 'context', 'lid', 'textgroup'));
    $sql_query->fields('t', array('translation', 'language'));

    switch ($query['translation']) {
      case 'translated':
        $sql_query->condition('t.translation', '%' . db_like($query['string']) . '%', 'LIKE');

        break;

      case 'untranslated':
        $sql_query->condition(db_and()
                ->condition('s.source', '%' . db_like($query['string']) . '%', 'LIKE')
                ->isNull('t.translation'));

        break;

      case 'all':
      default:
        $condition = db_or()
            ->condition('s.source', '%' . db_like($query['string']) . '%', 'LIKE');

        if ($query['language'] != 'en') {
          $condition->condition('t.translation', '%' . db_like($query['string']) . '%', 'LIKE');
        }

        $sql_query->condition($condition);

        break;
    }

    $sql_query->condition('s.textgroup', GLOBALLINK_ENTITY_TYPE_WEBFORM);
    $sql_query->condition('s.context', '%email%', 'NOT LIKE');
    $sql_query->condition('s.context', '%#%', 'LIKE');
    $sql_query->groupBy('lid')->extend('TableSort');

    $sort_order = isset($_GET['sort']) ? $_GET['sort'] : '';

    if ($sort_order) {
      $sql_query->orderBy('source', $_GET['sort']);
    }

    $result = $sql_query->execute();
    $webform_results = array();

    foreach ($result as $item) {
      array_push($webform_results, $item);
    }

    $locales = globallink_get_mapped_drupal_locales(FALSE);

    $rows = array();
    $count = 0;

    foreach ($webform_results as $webform) {
      $title = $webform->source;

      $results = db_select('webform_component', 'wc')
          ->fields('wc')
          ->condition('name', $title, '=')
          ->execute();

      foreach ($results as $result) {
        $nid = $result->nid;
      }

      $results_p = db_select(GLOBALLINK_ENTITY_TYPE_NODE, 'n')
          ->fields('n')
          ->condition('nid', $nid, '=')
          ->execute();

      foreach ($results_p as $result_p) {
        $title = $result_p->title;
      }

      $active = '';
      $active_arr = globallink_webform_get_active_submission_by_id($webform->lid);

      if (!empty($active_arr)) {
        foreach ($active_arr as $active_row) {
          $l_title = globallink_format_display_string($active_row->submission);
          $l_href = 'admin/globallink-translations/workbench/all/' . $active_row->submission_rid . '/' . $active_row->target_lang_code;
          $l_options = array();
          $active .= l($l_title, $l_href, $l_options) . '&nbsp;&nbsp;- &nbsp;&nbsp;' . $active_row->sub_target_lang_name . ' <BR/> ';
        }
      }

      if (!isset($rows[$webform->lid])) {
        $rows[$webform->lid] = array(
          'parent' => $title,
          'string' => check_plain(truncate_utf8($webform->source, 150, FALSE, TRUE)) . ' <br /> ',
          'languages' => $active,
          'nid' => $nid,
          'preview' => globallink_webform_make_preview_link('Preview', $webform->lid),
        );
      }

      $count++;
    }

    $cart_count = get_cart_count();

    globallink_add_cart_form_elements($form, $header, $rows, $default_checked, $cart_count);

    if ($count > 0) {
      $form['pager'] = array('#markup' => theme('pager'));
    }
  }
  
  // Very important
  form_load_include($form_state, 'inc', 'globallink', 'globallink_send_translations');

  return $form;
}

/**
 * Validates webform form input.
 */
function globallink_webform_dashboard_form_validate($form, &$form_state) {
  return;
}

/**
 * Handles webform form submission.
 */
function globallink_webform_dashboard_form_submit($form, &$form_state) {
  return;
}

/**
 * Function to create Preview link
 */
function globallink_webform_make_preview_link($link_text = '', $id) {
  drupal_add_js(array(
    'my-modal-style' => array(
      'modalSize' => array(
        'type' => 'fixed',
        'width' => 700,
        'height' => 400,
      ),
      'animation' => 'fadeIn',
    ),
      ), 'setting');

  // Set a default value if no text in supplied.
  if (empty($link_text)) {
    $link_text = 'Preview';
  }

  return '<div id="preview-link" style="text-align: left;" title="Click to Preview">' . l($link_text, 'admin/globallink-translations/dashboard/webform/preview/' . $id . '/nojs', array('attributes' => array('class' => 'ctools-use-modal ctools-modal-my-modal-style'))) . '</div>';
}

/**
 * Helper function
 */
function globallink_webform_preview($ajax) {
  if ($ajax) {
    ctools_include('ajax');
    ctools_include('modal');

    $form_state = array(
      'ajax' => TRUE,
      'title' => t('Preview'),
    );

    // Use ctools to generate ajax instructions for the browser to create a form in a modal popup.
    $output = ctools_modal_form_wrapper('globallink_webform_preview_content', $form_state);

    // If the form has been submitted, there may be additional instructions such as dismissing the modal popup.
    if (!empty($form_state['ajax_commands'])) {
      $output = $form_state['ajax_commands'];
    }

    // Return the ajax instructions to the browser via ajax_render().
    print ajax_render($output);
    drupal_exit();
  }
  else {
    return drupal_get_form('globallink_webform_preview_content');
  }
}

/**
 * Helper function
 */
function globallink_webform_preview_content() {
  $id = arg(5);
  $output = "";

  $query = db_select('locales_source', 'ls')
      ->fields('ls')
      ->condition('lid', $id, '=');
  $result = $query->execute();
  $result = $result->fetchAll();

  if (count($result) > 0) {
    $output .= "<table><tr><th>Fields</th><th>Source Contents</th></tr>";
    foreach ($result as $val) {
      $output .= "<tr><td><strong>Webform</strong></td><td>" . check_plain(truncate_utf8($val->source, 150, FALSE, TRUE)) . "</td></tr>";
    }
    $output .= "</table>";
  }

  $form['preview'] = array(
    '#markup' => $output,
  );

  return $form;
}
